/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.io.saving;

import java.util.Enumeration;

import net.noisetube.config.Preferences;
import net.noisetube.core.Pausable;
import net.noisetube.core.NTClient;
import net.noisetube.model.Saveable;
import net.noisetube.model.Track;
import net.noisetube.ui.SaverUI;
import net.noisetube.util.Logger;

/**
 * NoiseTube track and measurement saving class (abstract)
 * 
 * @author mstevens
 *
 */
public abstract class Saver implements Pausable
{
	
	protected Logger log = Logger.getInstance();
	protected NTClient ntClient;
	protected Preferences preferences;
	
	protected volatile boolean running = false;
	protected volatile boolean paused = false;
	private volatile String status;
	
	protected SaverUI ui;
	protected Track track;	
	
	public Saver(Track track)
	{
		this.track = track;
		ntClient = NTClient.getInstance();
		preferences = NTClient.getInstance().getPreferences();
	}
	
	public boolean isRunning()
	{
		return running;
	}
	
	public boolean isPaused()
	{
		return paused;
	}
	
	public abstract void save(Saveable saveable);
	
	protected abstract void enableBatchMode();
	
	public void saveBatch(Enumeration savables)
	{
		enableBatchMode(); //!!! 
		while(savables.hasMoreElements())
			save((Saveable) savables.nextElement());
	}
	
	public void setTrack(Track track)
	{
		if(isRunning())
			throw new IllegalStateException("Cannot replace track while saver is still running");
		this.track = track;
	}
	
	public void setUI(SaverUI ui)
	{
		this.ui = ui;
	}
	
	protected void setStatus(String message)
	{
		status = message;
		if(ui != null)
			ui.updated(this, message);
	}

	/**
	 * @return the status
	 */
	public String getStatus()
	{
		return status;
	}

}
