/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.model;

import java.util.Vector;

import net.noisetube.tagging.TagsHelper;
import net.noisetube.util.CustomStringBuffer;
import net.noisetube.util.URLUTF8Encoder;

/**
 * TaggedInterval class
 * 
 * Indexes are 0-based and the interval _includes_ the begin and end index
 * 
 * @author mstevens, sbarthol
 *
 */
public class TaggedInterval implements Saveable
{

	private int beginIdx;
	private int endIdx;
	private boolean automatic;
	private Vector tags;
	
	public TaggedInterval(int beginIdx, int endIdx, Vector tags)
	{
		this(beginIdx, endIdx, tags, false);
	}
	
	public TaggedInterval(int beginIdx, int endIdx, Vector tags, boolean automatic)
	{
		if(beginIdx > endIdx)
			throw new IllegalArgumentException("endIndex cannot come before beginIndex");
		this.beginIdx = beginIdx;
		this.endIdx = endIdx;
		this.automatic = automatic;
		this.tags = tags;
	}
	
	/**
	 * @return the beginIdx
	 */
	public int getBeginIdx()
	{
		return beginIdx;
	}
	
	/**
	 * @return the endIdx
	 */
	public int getEndIdx()
	{
		return endIdx;
	}
	
	/**
	 * @return the automatic
	 */
	public boolean isAutomatic()
	{
		return automatic;
	}

	/**
	 * @return the tags
	 */
	public Vector getTags()
	{
		return tags;
	}
	
	/* (non-Javadoc)
	 * @see net.noisetube.model.SavableModel#toXML()
	 */
	public String toXML()
	{
		CustomStringBuffer bff = new CustomStringBuffer();
		bff.append("<taggedInterval beginIndex=\"" + beginIdx + "\"");
		bff.append(" endIndex=\"" + endIdx + "\"");
		if(automatic)
			bff.append(" autotags=\"" + TagsHelper.getSavableTagsString(tags) + "\"");
		else
			bff.append(" tags=\"" + TagsHelper.getSavableTagsString(tags) + "\"");
		bff.append("/>");
		return bff.toString();
	}
	
	/* 
	 * Format: [beginIdx,endIdx,"tags space separated",automatic]
	 * 
	 * (non-Javadoc)
	 * @see net.noisetube.model.SavableModel#toJSON()
	 */
	public String toJSON()
	{
		StringBuffer bff = new StringBuffer("[");
		bff.append(beginIdx + ",");
		bff.append(endIdx + ",");
		bff.append("\"" + TagsHelper.getSavableTagsString(tags) + "\",");
		bff.append(Boolean.toString(automatic));
		bff.append("]");
		return bff.toString();
	}

	/* (non-Javadoc)
	 * @see net.noisetube.model.SavableModel#toUrl()
	 */
	public String toUrl()
	{
		StringBuffer bff = new StringBuffer();
		bff.append("&beginIdx=" + beginIdx);
		bff.append("&endIdx=" + endIdx);
		if(automatic)
			bff.append("&autotag=" + URLUTF8Encoder.encode(TagsHelper.getSavableTagsString(tags)));
		else
			bff.append("&tag=" + URLUTF8Encoder.encode(TagsHelper.getSavableTagsString(tags)));
		return bff.toString();
	}
	
}
