/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.tagging;

import java.util.Enumeration;

import net.noisetube.core.Processor;
import net.noisetube.model.NTMeasurement;
import net.noisetube.model.Track;

/**
 * Simple single measurement tagger for human tags
 * 
 * This processor accepts a tagsString and a timestamp and applies the tags to the measurement
 * in the track buffer which was made closest to timestamp + a configurable delay. 
 * 
 * @author mstevens, maisonneuve
 */
public class SingleMeasurementTagger implements Processor
{
	
	public static int TAGGING_DELAY_MS = 4000; //4 seconds

	private String tags = null;
	private long tagTimeStamp;


	/**
	 * @param tags string of tags (comma separated)
	 * @param tagTimeStamp time when the user started typing the tag(s)
	 */
	public void setTags(String tags, long tagTimeStamp)
	{
		this.tags = tags;
		this.tagTimeStamp = tagTimeStamp;
	}

	public void process(NTMeasurement newMeasurement, Track track)
	{
		if(tags != null)
		{
			//Associate tag with measurement made TAGGING_DELAY ms before tagTimeStamp (= when the user started typing the tag):
			Enumeration mEnum = track.getMeasurementsNewestFirst();
			while(mEnum.hasMoreElements())
			{
				NTMeasurement m = (NTMeasurement) mEnum.nextElement();
				if(m.getTimeStamp().getTime() + TAGGING_DELAY_MS < tagTimeStamp)
				{
					m.addUserTags(tags);
					break;
				}
			}
			tags = null;
			tagTimeStamp = 0;
		}
	}

	public void reset()
	{
		tags = null;
		tagTimeStamp = 0;	
	}

	public String getName()
	{
		return "Human single measurement tagging";
	}
	
}
