/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation; Android version)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 *  Android port by Vrije Universiteit Brussel (BrusSense team), 2010-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.location.android;

import java.util.Vector;

import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import net.noisetube.location.GeoTagger;
import net.noisetube.model.NTLocation;
import net.noisetube.ui.android.MainActivity;

/**
 * @author sbarthol, mstevens
 * 
 */
public class AndroidGeoTagger extends GeoTagger implements LocationListener
{
	
	private LocationManager locationManager;
	private boolean gpsEnabled = false;

	public AndroidGeoTagger(LocationManager locationManager)
	{
		this.locationManager = locationManager;
	}

	@Override
	public boolean startGPS()
	{
		if(running && preferences.isUseGPS())		
		{
			MainActivity.getInstance().runOnUiThread(new Runnable() //!!! to avoid looper problems
			{
				public void run()
				{
					try
					{				
						if(locationManager != null && locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER))
						{	/* requestLocationUpdates (String provider, long minTime, float minDistance, LocationListener listener) */
							locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0 /* ms */, 3.5f /* meter */, AndroidGeoTagger.this);
							//setLastLocation(new NTLocation(new AndroidNTCoordinates(locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER))));
							gpsEnabled = true;
						}
					}
					catch(Exception e)
					{
						log.error(e, "startGPS");
						gpsEnabled = false;
					}
				}
			});
		}
		return gpsEnabled;
	}
	
	@Override
	public boolean isGPSEnabled()
	{
		return gpsEnabled;
	}
	
	@Override
	public void stopGPS()
	{
		try
		{
			gpsEnabled = false;
			locationManager.removeUpdates(this);
		}
		catch(Exception ignore) {}
	}
	
	public boolean isAutoModeEnabled()
	{
		return gpsEnabled;
	}

	public void onLocationChanged(Location location)
	{	
		if(running && gpsEnabled)
			setLastLocation(new NTLocation(new AndroidNTCoordinates(location))); //When a new location is found it needs to be stored		
	}

	public void onStatusChanged(String provider, int status, Bundle extras)
	{
		switch(status)
		{
			case LocationProvider.OUT_OF_SERVICE : disableGPS(); break;
			case LocationProvider.AVAILABLE :
				if(gotFix())
					setGPSState(GPS_STATE_GOT_FIX);
				else
					setGPSState(GPS_STATE_WAITING);
				break;
			case LocationProvider.TEMPORARILY_UNAVAILABLE : setGPSState(GeoTagger.GPS_STATE_SUSPENDED); break;
		}
	}

	public void onProviderDisabled(String provider)
	{
		gpsEnabled = false;
	}

	public void onProviderEnabled(String provider)
	{
		gpsEnabled = true;
	}
	
	
//	//TEST SUITE FOR INTERPOLATION BUFFER:
//	static Vector testCoords; static {
//	testCoords = new Vector();
//	for(int i = 0; i < 10; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84351494838667, 2.347631728280715, 0));
//	for(int i = 0; i < 7; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84358019341609, 2.348121625911297, 0));
//	for(int i = 0; i < 13; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84302743833563, 2.348187635222634, 0));
//	for(int i = 0; i < 9; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84280123409627, 2.348206402108299, 0));
//	for(int i = 0; i < 5; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84301475858108, 2.349128511479379, 0));
//	for(int i = 0; i < 11; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84311671676204, 2.349599211777118, 0));
//	for(int i = 0; i < 2; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84376145818703, 2.34942931441376, 0));
//	for(int i = 0; i < 8; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84439838833931, 2.349317045072576, 0));
//	for(int i = 0; i < 3; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84449440729303, 2.349227473277136, 0));
//	for(int i = 0; i < 2; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84440606913172, 2.348143241962966, 0));
//	for(int i = 0; i < 9; i++)
//		testCoords.addElement(new AndroidNTCoordinates(48.84361331326151, 2.348104480544591, 0));
//	}
//	static int c = -1;
//
//	public void start()
//	{
//		c = -1;
//		super.start();
//	}
//	
//	public NTLocation getLastLocation()
//	{
//		c++;
//		if(c >= testCoords.size())
//			return null;
//		return new NTLocation((AndroidNTCoordinates)testCoords.elementAt(c));
//	}
}
