/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation; Android version)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 *  Android port by Vrije Universiteit Brussel (BrusSense team), 2010-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.ui.android;

import java.util.HashMap;
import java.util.Iterator;

import net.noisetube.R;
import net.noisetube.audio.calibration.Calibration;
import net.noisetube.config.Preferences;
import net.noisetube.core.NTClient;
import net.noisetube.core.android.AndroidNTService;
import net.noisetube.model.NTMeasurement;
import net.noisetube.model.Track;
import net.noisetube.ui.TrackUI;
import net.noisetube.ui.android.LoginActivity;
import net.noisetube.ui.android.Toaster;
import net.noisetube.ui.android.tabs.Tab;
import net.noisetube.ui.android.tabs.TabLoadingActivity;
import net.noisetube.ui.android.tabs.TabGraphActivity;
import net.noisetube.ui.android.tabs.TabMapActivity;
import net.noisetube.util.Logger;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.TabActivity;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TabHost;
import android.widget.TextView;

// #AMBIENTTALK
import android.os.AsyncTask;
import edu.vub.at.IAT;
import edu.vub.at.android.util.IATAndroid;
import edu.vub.at.android.util.IATSettings;
import edu.vub.at.android.util.IATSettings.IATOptions;
import edu.vub.at.game.NoiseMappingGameAssetInstaller;
import edu.vub.at.game.interfaces.ATNoiseMappingGame;
import edu.vub.at.game.interfaces.JNoiseMappingGame;

/**
 * @author sbarthol, mstevens
 *
 * This class compromises the main GUI of the app
 * Extensions for DMPP project are tagged as #AMBIENTTALK 
 */
public class MainActivity extends TabActivity implements TrackUI,JNoiseMappingGame
{

	//STATICS-------------------------------------------------------

	private static final String STATUS_PAUSED = "Paused";
	private static final String STATUS_MEASURING = "Measuring";
	//Instance
	static private MainActivity instance;

	/**
	 * @return the instance
	 */
	static public MainActivity getInstance()
	{
		return instance;
	}
	
	// #AMBIENTTALK
	private static IAT iat;
	
	// #AMBIENTTALK
	private static final int _ASSET_INSTALLER_ = 0;

	// #AMBIENTTALK
	public class StartIATTask extends AsyncTask<Void, String, Void> {
        @Override
        protected Void doInBackground(Void... arg0) {
            try {
            	IATOptions iatOptions = IATSettings.getIATOptions(MainActivity.this);
				iatOptions.networkName_ = "AmbientTalk";
                iat = IATAndroid.create(MainActivity.this, iatOptions);
                
                this.publishProgress("Loading AmbientTalk code");
                game = (ATNoiseMappingGame)iat.evalAndWrap("/.game.noiseMappingGame.makeNoiseMappingGame()", ATNoiseMappingGame.class);
                game.registerActivity(MainActivity.this);
    			// #AMBIENTTALK
    			Track track = service.getTrack();
    			track.addProcessor(game);
    			Log.v("Noise Mapping Game", "AmbientTalk started");
            } catch (Exception e) {
            	Log.e("AmbientTalk", "Could not start IAT", e);
            }
            return null;
        }
	}
	
	//DYNAMICS------------------------------------------------------
	private Logger log;
	/*private AndroidPreferences preferences;*/
	/*private Track track = null;*/
	private AndroidNTService service = null;

	private Dialog splashScreen;
	private TabHost tabHost;
	private HashMap<String, Tab> tabs = new HashMap<String, Tab>();
	private boolean duplicateActivity = false;

	// #AMBIENTTALK
	private ATNoiseMappingGame game;
	
	/*
	 * We keep this here instead of in the map tab such that we can show the whole track including places
	 * visited before the map tab was first opened (the tab objects are only created when they are first opened)
	 */
	private NoiseLevelOverlay noiselevelOverlay;

	private Button pause_resumeButton;
	private Button start_stopButton;
	
	private TextView lblStatus;

	private boolean calibrationToastShown = false;
	private boolean showingAuxActivityOrDialog = false; //to avoid pausing when about/pref's is shown and pauseOnBackground is enabled
	/*
	private boolean pausedForBackground = false;
	private boolean pausedForCall = false;
	 */

	private void startService()
	{
		Intent serviceIntent = new Intent(getApplicationContext(), AndroidNTService.class);
		startService(serviceIntent);
	}

	/** Called when the activity is first created.
	 * @param savedInstanceState
	 */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{	
		super.onCreate(savedInstanceState);
		if(instance!=null)
		{
			//Necessary branch, since the notification button may start a new instance of MainActivity.	
			duplicateActivity = true;
			finish();
			//Note that return is probably useless, but here for safety measures.
			return;
		}
		instance = this; //!!!

		try
		{
			//Get log instance:
			log = Logger.getInstance();

			tabHost = getTabHost();

			//In case the service was never created before, make sure to create the service first
			if(AndroidNTService.getInstance() == null)
			{
				startService();

				/* On Android v2.1 (and probably earlier versions too, but we do not support those) a crash happens
				 * when no tabs are set before OnCreate ends.
				 * 		Cfr.: http://stackoverflow.com/questions/2020046/npe-when-drawing-tabwidget-in-android-only-on-htc-magic
				 * Solution: we use a dummy tab as a temporary place holder while loading the AndroidNTClient */
				addTab(TabLoadingActivity.class, getResources().getDrawable(R.drawable.noisetube_dialog_icon), "Loading");

				//show splash screen while loading
				showSplashScreen();
			}
			else
			{
				service = AndroidNTService.getInstance();
				service.bindGui(this);
				initializeGUI();
			}

			//new InitializeAndroidNTClient().execute(this); //will call initializeActivity when successful


		}
		catch (Exception e)
		{
			log.error(e, "MainActivity.onCreate()");
			removeSplashScreen();
			showErrorDialog(e);
		}
	}
	
	// #AMBIENTTALK
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		Log.v("Noise Mapping Game", "Return of Asset Installer activity");
		switch (requestCode) {
		case (_ASSET_INSTALLER_):
			if (resultCode == Activity.RESULT_OK) {
				new StartIATTask().execute((Void)null);
			}
			break;
		}
	}
	
	// #AMBIENTTALK
	/** To initialize AmbientTalk
	 */
	private void initializeAmbientTalk() {
		if (iat == null) {
			// install AmbientTalk provider factory and discoverer factory
			Intent i = new Intent(this, NoiseMappingGameAssetInstaller.class);
			startActivityForResult(i, _ASSET_INSTALLER_);

		}
	}
	
	// #AMBIENTTALK
	// Methods implementing the JWeScribble interface
	/*public JNoiseMappingGame registerATApp(ATNoiseMappingGame aGame) {
		game = aGame;
		return this;
	}*/

	public void serviceCreated()
	{
		service = AndroidNTService.getInstance();
		service.bindGui(this);
		service.initializeService();
	}

	private void initializeGUI()
	{
		//GUI:
		setContentView(R.layout.mainactivity);
		noiselevelOverlay = new NoiseLevelOverlay(); //map overlay
		initializeTabs();

		// Bind the pause_resume button:
		pause_resumeButton = (Button) findViewById(R.id.pauseButton);
		pause_resumeButton.setOnClickListener(pause_resumeButtonListener);

		// Bind the start_stop button:
		start_stopButton = (Button) findViewById(R.id.startstopButton);
		start_stopButton.setOnClickListener(start_stopButtonListener);
		
		// Status label:
		lblStatus = (TextView) findViewById(R.id.lblMeasuringStatus);
		lblStatus.setTextColor(Color.CYAN);

		if(service != null)
		{
			if(service.isPaused())
				pause_resumeButton.setText("Resume");
		}
	}

	public void serviceInitialized()
	{
		initializeGUI();

		//Show gui by removing splash screen that is shown on top of it:
		removeSplashScreen(); //!!!

		//Display main form with measurement (or login form, if needed) and start measuring
		startMeasuring();
		
		// #AMBIENTTALK
		initializeAmbientTalk();
		//add processor using service variable
	}

	private void addTab(Class tabClass, Drawable icon, String title)
	{
		//Initialize a TabSpec for each tab and add it to the TabHost
		tabHost.addTab(tabHost.newTabSpec(tabClass.getSimpleName()).setIndicator(title, icon).setContent(new Intent().setClass(this, tabClass)));
	}

	/*
	 * Called from Tab instances
	 */
	public void registerTab(Tab tab)
	{
		if(!tabs.containsKey(tab.getClass().getSimpleName()))
			tabs.put(tab.getClass().getSimpleName(), tab);
	}

	/**
	 * This function can be used to renew the tabs, for example, when the GPS is being disabled (in
	 * this case the map tab is useless).
	 * 
	 * TODO hide map tab when gps is disabled: http://stackoverflow.com/questions/2645703/hide-tab-in-tabactivity
	 */
	private void initializeTabs()
	{
		Resources res = getResources(); //Resource object to get Drawables
		tabHost = getTabHost(); //just in case it changed since onCreate (can that happen?)
		tabHost.clearAllTabs();
		tabs.clear();	

		addTab(TabGraphActivity.class, res.getDrawable(R.drawable.ic_tab_graph), "Graph");

		//Only add the map when the gps is enabled in preferences.
		if(NTClient.getInstance().getPreferences().isUseGPS())
			addTab(TabMapActivity.class, res.getDrawable(R.drawable.ic_tab_map), "Map");

		//Spectrum disabled
		//addTab(TabSpectrumActivity.class, res.getDrawable(R.drawable.ic_tab_spec), "Spectrum");

		//addTab(TabTagActivity.class, res.getDrawable(R.drawable.ic_tab_tag), "Tag");

		//Tab changed event:
		tabHost.setOnTabChangedListener(new TabHost.OnTabChangeListener()
		{
			public void onTabChanged(String tag)
			{
				tabs.get(tag).refresh();

				//tab title colors:
				updateTabTitleColors();
			}
		});

		updateTabTitleColors();
	}

	private void updateTabTitleColors()
	{
		for(int i=0; i< tabHost.getTabWidget().getChildCount();i++) 
		{ 
			TextView tv = (TextView) tabHost.getTabWidget().getChildAt(i).findViewById(android.R.id.title); //Unselected Tabs
			tv.setTextColor(Color.WHITE);
		} 
		TextView tv = (TextView) tabHost.getCurrentTabView().findViewById(android.R.id.title); //for Selected Tab
		tv.setTextColor(Color.BLACK);
	}

	public void startMeasuring()
	{
		// First show login form if needed:
		if(service.loginRequired())
			showLoginActivity();
		else {
			service.startMeasuring();
		}
	}

	/**
	 * Assumes there is a track and it is running (not checked here but at all calls)
	 * 
	 * @param restartAfterwards
	 */
	public void stopMeasuring(boolean restartAfterwards)
	{
		service.pauseMeasuring(true); //"pre-stop" pause (only stops AudioComponent)
		final TrackSummaryDialog trackSummaryDialog = new TrackSummaryDialog(this, service.getTrack(), (service.getTrack().getSaver() != null), restartAfterwards);
		showingAuxActivityOrDialog = true;
		trackSummaryDialog.show();
		if(service.getTrack().getSaver() != null)
		{
			(new Thread()
			{
				public void run()
				{
					service.stopMeasuring(false); //will block while saving completes or waiting time runs out
					runOnUiThread(new Runnable()
					{
						public void run()
						{
							trackSummaryDialog.stopWaiting(!service.getTrack().getSaver().isRunning());
						}
					});	
				}
			}).start();
		}
		else
			service.stopMeasuring(false);
	}

	public void showLoginActivity()
	{
		startActivity(new Intent(MainActivity.this, LoginActivity.class));
	}

	private void showPreferences()
	{
		showingAuxActivityOrDialog = true;
		startActivity(new Intent(this, PreferencesActivity.class));
	}

	private void showAbout()
	{
		showingAuxActivityOrDialog = true;
		startActivity(new Intent(this, AboutActivity.class));
	}

	public void showGraph()
	{
		getTabHost().setCurrentTabByTag(TabGraphActivity.class.getSimpleName());
	}

	/**
	 * @param track
	 * @param newMeasurement
	 * @param savedMeasurement the measurement which has been removed from the track and saved to make room for the new one
	 */
	public void newMeasurement(final Track track, final NTMeasurement measurement, final NTMeasurement savedMeasurement)
	{
		runOnUiThread(new Runnable()
		{
			public void run()
			{
				//Add to noiseleveloverlay for the map:
				noiselevelOverlay.update(track, savedMeasurement);

				//Update tabs:
				String currentTabTag = tabHost.getCurrentTabTag();
				Iterator<Tab> tabIter = tabs.values().iterator();
				while(tabIter.hasNext())
				{
					Tab tab = tabIter.next();
					if(currentTabTag.equals(tab.getClass().getSimpleName()) || tab.mustRemainInformed())
						tab.update(measurement, track);
				}
			}
		});
	}

	public void measuringStarted(Track track)
	{
		runOnUiThread(new Runnable()
		{
			public void run()
			{
				pause_resumeButton.setEnabled(true);
				start_stopButton.setText("Stop");
				start_stopButton.setEnabled(true); //unblock button
				Iterator<Tab> tabIter = tabs.values().iterator();
				while(tabIter.hasNext())
					tabIter.next().start();
				//Toaster.displayShortToast("Measuring started");
				lblStatus.setText(STATUS_MEASURING);
				if(!calibrationToastShown)
				{
					Calibration calibration = NTClient.getInstance().getPreferences().getCalibration();
					if(calibration == null || calibration.getEffeciveCredibilityIndex() > Calibration.CREDIBILITY_INDEX_G)
						Toaster.displayToast("NoiseTube is currently not calibrated for your phone model.\nGeneric setting in use, measurements are likely inaccurate.\nNew calibration settings are downloaded automatically when they become available.");
					else if(calibration.getEffeciveCredibilityIndex() == Calibration.CREDIBILITY_INDEX_G)
						Toaster.displayToast("NoiseTube is currently not calibrated for your phone model.\nBrand default setting in use, measurements may be inaccurate.\nNew calibration settings are downloaded automatically when they become available.");
					else
						Toaster.displayToast("Using calibration setting for your phone model for best accuracy.");
					calibrationToastShown = true;
				}
			}
		});
	}

	public void measuringStopped(Track track)
	{
		runOnUiThread(new Runnable()
		{
			public void run()
			{
				pause_resumeButton.setEnabled(false);
				start_stopButton.setText("Start");
				start_stopButton.setEnabled(true); //unblock button
				Iterator<Tab> tabIter = tabs.values().iterator();
				while(tabIter.hasNext())
					tabIter.next().stop();
				//Toaster.displayShortToast("Measuring stopped"); //is already clear from summary dialog
				lblStatus.setText(""); //="stopped"
			}
		});
	}

	public void measuringPaused(Track track)
	{
		runOnUiThread(new Runnable()
		{
			public void run()
			{
				pause_resumeButton.setText("Resume");
				pause_resumeButton.setEnabled(true); //unblock button
				//Toaster.displayShortToast("Measuring paused");
				lblStatus.setText(STATUS_PAUSED);
			}
		});
	}

	public void measuringResumed(Track track)
	{
		runOnUiThread(new Runnable()
		{
			public void run()
			{
				pause_resumeButton.setText("Pause");
				pause_resumeButton.setEnabled(true); //unblock button
				//Toaster.displayShortToast("Measuring resumed");
				lblStatus.setText(STATUS_MEASURING);
			}
		});
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.mainmenu, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu)
	{
		super.onPrepareOptionsMenu(menu);
		menu.findItem(R.id.switchlogin).setVisible(service.getPreferences().getSavingMode() == Preferences.SAVE_HTTP);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item)
	{
		// Handle item selection
		switch (item.getItemId())
		{
		case R.id.preferences : showPreferences(); return true;
		case R.id.about: showAbout(); return true;
		case R.id.switchlogin:
		{
			service.getPreferences().setAccount(null); //= "log-out"
			if(service.isMeasuring())
				stopMeasuring(true);
			return true;
		}
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	private OnClickListener pause_resumeButtonListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			pause_resumeButton.setEnabled(false); //block button to avoid hammering (causing repeated pausing/resuming)
			service.pause_resume();
		}
	};

	private OnClickListener start_stopButtonListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			start_stopButton.setEnabled(false); //block button to avoid hammering (causing repeated starts/stops)
			if(service.isMeasuring())
				stopMeasuring(false);
			else
				startMeasuring();
		}
	};

	@Override
	public void onConfigurationChanged(Configuration newConfig)
	{
		super.onConfigurationChanged(newConfig);
		//Toaster.displayShortToast("Config changed");
		//do nothing...
	}

	@Override
	protected void onPause()
	{
		super.onPause();
		if(!showingAuxActivityOrDialog)
			pause();
		if(!isFinishing())
		{
			//TODO make sure the service is not decoupled when this activity is destroyed for "cleanup" (when run in background)
			if(service!=null)
				service.decouple();
		}
	}

	private void pause()
	{
		if(service != null)
			service.pauseMeasuringForBackground();
	}

	@Override
	protected void onResume()
	{
		super.onResume();
		resume();
		showingAuxActivityOrDialog = false;
		if(service!=null)
			service.bindGui(this);
	}

	private void resume()
	{
		if(service != null)
			service.resumeMeasuringFromBackground();
	}

	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		/* If this instance of MainActivity is destroyed because it is a duplicate (or in other words,
		 * because another version of the application has already existed) we must make sure not
		 * to remove anything related to the other instance or the service.
		 */
		if(isFinishing() && !duplicateActivity)
		{
			try
			{
					//Also make sure the appropriate static variables etc, stored in the tabs, are cleaned up
					instance = null;
					//TODO make sure the service is not destroyed when this activity is destroyed for "cleanup" (when run in background)
					service.destroy();
					service = null;
					// #AMBIENTTALK
					iat = null;
			}
			catch(Exception ignore) {}
		}
	}

	/**
	 * @return the noiselevelOverlay
	 */
	public NoiseLevelOverlay getNoiselevelOverlay()
	{
		return noiselevelOverlay;
	}

	/**
	 * Shows the splash screen over the full Activity
	 */
	protected void showSplashScreen()
	{
		showingAuxActivityOrDialog = true;
		splashScreen = new Dialog(this, R.style.DefaultTheme);
		splashScreen.setTitle(R.string.app_name);
		splashScreen.setContentView(R.layout.splashscreen);
		splashScreen.setCancelable(false);
		splashScreen.show();
	}

	/**
	 * Removes the Dialog that displays the splash screen
	 */
	protected void removeSplashScreen()
	{
		if(splashScreen != null)
		{
			splashScreen.dismiss();
			splashScreen = null;
		}
	}

	protected void showErrorDialog(Exception error)
	{
		Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(getString(R.string.app_name) + " error");
		builder.setIcon(R.drawable.noisetube_dialog_icon);
		builder.setMessage(error.getMessage());
		builder.setOnCancelListener(new DialogInterface.OnCancelListener()
		{
			public void onCancel(DialogInterface dialog)
			{
				finish();
			}
		});
		builder.setNeutralButton("Exit", new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int which)
			{
				finish();
			}
		});
		showingAuxActivityOrDialog = true;
		builder.show();
	}

	/**
	 * Displays the tagging screen with as "taggable set of measurements"
	 * the measurements in the track between the measurement at "numberFirstM" and the measurement at "numberLastM".
	 * measurements at numberFirstM and numberLastM are included.
	 * @param firstIdXOnGraph location of first measurement, start counting from 0 (0 = for denoting the leftmost measurement currently on graph)
	 * @param lastIdxOnGraph location of last measurement, start counting from 0 (0 = for denoting the leftmost measurement currently on graph)
	 */
	public void bringUpTagScreen(final int firstIdXOnGraph, final int lastIdxOnGraph)
	{
		if(service.isMeasuring())
		{
			Track track = service.getTrack();
			final TagDialog tagDialog = new TagDialog(	this,
														track,
														//calculate index of first to-be-tagged measurement within track:
														track.getStatistics().getNumMeasurements() - (track.getBufferSize()-firstIdXOnGraph),
														//calculate index of last to-be-tagged measurement within track:
														track.getStatistics().getNumMeasurements() - (track.getBufferSize() - Math.min(lastIdxOnGraph, track.getBufferSize() - 1)));
			tagDialog.show();
			if(service.getPreferences().isPauseDuringTagTyping())
				service.pauseMeasuring(false);
		}
		//TODO preference "pause on tagging"

		/*
		try
		{
			runOnUiThread(new Runnable()
			{
				public void run()
				{

					String tagTabTag = TabTagActivity.class.getSimpleName();
					tabHost.setCurrentTabByTag(tagTabTag);
					//tab title colors:
					updateTabTitleColors();
					tabHost.postInvalidate();

					((TabTagActivity) tabs.get(TabTagActivity.class.getSimpleName())).fixedTagging();
					((TabTagActivity) tabs.get(TabTagActivity.class.getSimpleName())).tagMeasurementsSet(numberFirstM, numberLastM);
				}
			});	

		} catch(Exception e) {
		}
	*/
	}

}