/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation; Android version)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 *  Android port by Vrije Universiteit Brussel (BrusSense team), 2010-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.ui.android;

import net.noisetube.R;
import net.noisetube.config.Preferences;
import net.noisetube.config.android.*;
import net.noisetube.core.NTClient;
import net.noisetube.core.android.AndroidNTService;
import android.app.Activity;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.RadioButton;

/**
 * @author sbarthol, mstevens
 *
 */
public class PreferencesActivity extends Activity
{
	
	private AndroidNTService service;
	
	private AndroidPreferences preferences;
	private AndroidDevice device;
	
	private boolean changesMade = false;
	private boolean requiresStop = false;
	
	private RadioButton  radioHTTP;
	private RadioButton  radioFile;
	private RadioButton  radioNone;

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.preferencesactivity);
		service = AndroidNTService.getInstance();
		preferences = (AndroidPreferences) NTClient.getInstance().getPreferences();
		device = (AndroidDevice) NTClient.getInstance().getDevice();
		
		//Get UI controls:
		CheckBox checkGPS = (CheckBox) findViewById(R.id.usegps);
		CheckBox checkMemoryCard = (CheckBox) findViewById(R.id.memorycard);
		radioHTTP = (RadioButton) findViewById(R.id.usehttp);
		radioFile = (RadioButton) findViewById(R.id.usefile);
		radioNone = (RadioButton) findViewById(R.id.usenone);
		CheckBox checkPauseOnBackground = (CheckBox) findViewById(R.id.pauseWhenInBackground);
		
		//Set controls to current preferences state
		//	GPS
		checkGPS.setChecked(preferences.isUseGPS());
		//	Memory card
		checkMemoryCard.setChecked(preferences.isPreferMemoryCard());
		//	Saving modes...
		switch(preferences.getSavingMode())
		{
			case Preferences.SAVE_HTTP : radioHTTP.setChecked(true); break;
			case Preferences.SAVE_FILE : radioFile.setChecked(true); break;
			case Preferences.SAVE_NO : radioNone.setChecked(true); break;
			default : radioNone.setChecked(true);
		}
		radioHTTP.setEnabled(device.supportsInternetAccess()); //pings the server!
		radioFile.setEnabled(device.supportsFileAccess());
		if(!radioHTTP.isEnabled() && radioHTTP.isChecked())
			radioFile.setChecked(true); //http is unavailable but is selected --> switch to file
		if(!radioFile.isEnabled() && radioFile.isChecked())
			radioNone.setEnabled(true); //file is unavailable but is selected --> switch to none
		//	Pause
		checkPauseOnBackground.setChecked(preferences.isPauseWhenInBackground());
		
		//Configure listeners
		checkGPS.setOnClickListener(gpsListener);
		checkMemoryCard.setOnClickListener(memoryListener);
		radioHTTP.setOnClickListener(httpListener);
		radioFile.setOnClickListener(fileListener);
		radioNone.setOnClickListener(noneListener);
		checkPauseOnBackground.setOnClickListener(pauseOnBackgroundListener);

		//Exiting this screen can be done by using standard Android return button or by custom Done button
		Button doneButton = (Button) findViewById(R.id.btnPreferencesOK);
		doneButton.setOnClickListener(doneButtonListener);
	}

	private OnClickListener gpsListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			CheckBox gpsBox = (CheckBox) v;
			preferences.setUseGPS(gpsBox.isChecked());
			NTClient.getInstance().getGeoTagger().toggleGPS();
			changesMade = true;
			//requiresStop = false;
		}
	};

	private OnClickListener memoryListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			CheckBox memoryBox = (CheckBox) v;
			preferences.setPreferMemoryCard(memoryBox.isChecked());
			changesMade = true;
			requiresStop = true;
		}
	};
	
	private OnClickListener pauseOnBackgroundListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			CheckBox memoryBox = (CheckBox) v;
			preferences.setPauseWhenInBackground(memoryBox.isChecked());
			changesMade = true;
			//requiresStop = false;
		}
	};

	private OnClickListener httpListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			if(preferences.getSavingMode() != Preferences.SAVE_HTTP || !preferences.isAlsoSaveToFileWhenInHTTPMode())
			{
				preferences.setSavingMode(Preferences.SAVE_HTTP);
				preferences.setAlsoSaveToFileWhenInHTTPMode(true);
				changesMade = true;
				requiresStop = true;
			}
		}
	};

	private OnClickListener fileListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			if(preferences.getSavingMode() != Preferences.SAVE_FILE)
			{
				preferences.setSavingMode(Preferences.SAVE_FILE);
				changesMade = true;
				requiresStop = true;
			}
		}
	};

	private OnClickListener noneListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			if(preferences.getSavingMode() != Preferences.SAVE_NO)
			{
				preferences.setSavingMode(Preferences.SAVE_NO);
				changesMade = true;
				requiresStop = true;
			}
		}
	};

	private OnClickListener doneButtonListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			finish();
		}
	};
	
	@Override
	public void onConfigurationChanged(Configuration newConfig)
	{
	    super.onConfigurationChanged(newConfig);
	    //Toaster.displayShortToast("Config changed");
	    //do nothing...
	}
	
	@Override
	protected void onPause()
	{
		super.onPause();
		if(!isFinishing())
			service.pauseMeasuringForBackground(); //to pause measuring if user wants that
	}

	@Override
	protected void onResume()
	{
		super.onPause();
		if(!isFinishing())
			service.resumeMeasuringFromBackground(); //to resume measuring
	}

	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		if(isFinishing() && changesMade)
		{
			preferences.saveToStorage();
			if(service.isMeasuring() && requiresStop)
			{
				service.stopMeasuring(true); //!!! will automatically restart with new settings (and show login if needed)
				requiresStop = false;
			}
			changesMade = false;
		}
	}
}
