/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation; Android version)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 *  Android port by Vrije Universiteit Brussel (BrusSense team), 2010-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.ui.android.tabs;

import java.util.Enumeration;
import java.util.Vector;

import net.noisetube.R;
import net.noisetube.core.android.AndroidNTService;
import net.noisetube.io.saving.Saver;
import net.noisetube.model.NTMeasurement;
import net.noisetube.model.TaggedInterval;
import net.noisetube.model.Track;
import net.noisetube.ui.android.MainActivity;
import net.noisetube.ui.android.Toaster;
import net.noisetube.util.CyclicQueue;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * This Activity is used to tag the last 1 to X (default: 10) measurements with a custom tag(s). There where some design choices made: 1. You can tag the last 1
 * to 10 measurements that where measured BEFORE you opened this tab. 2. If you do not close this tab and keep on tagging, you'll tag the same measurements. 3.
 * The measuring isn't automatically being paused, since this is also a tab, and should therefore not act radically different from other tabs. The pause button
 * is also always available. 4. We do need to take into account that when the measuring isn't paused, the locally stored measurements might get outdated. In
 * other word, when we wait to long with adding the tag, they may have been thrown out the track and may have already be saved, without tag.
 * 
 * TODO dropdown box of suggested &/or previously used tags
 * 
 * @author sbarthol, mstevens
 */
public class TabTagActivity extends Activity implements Tab
{

	private static int MAX_TAGGABLE_MEASUREMENTS_SERIES = 10;

	private MainActivity mainActivity;
	private AndroidNTService service;

	private EditText txtTag;
	private TextView txtNumberOfMeasurements;
	private TextView txtHowManyQuestion;
	private Button btnTag;
	private int numberOfMeasurementsToTag = MAX_TAGGABLE_MEASUREMENTS_SERIES; // default at max
	private boolean needsNewTrack = false;
	private CyclicQueue tagQueue = new CyclicQueue();

	private Button btnPlus;
	private Button btnMin;
	private int numberFirstM = 0;
	private int numberLastM = 0;

	//private static boolean fixedTagging = false;;

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		mainActivity = MainActivity.getInstance();
		if(mainActivity != null)
			mainActivity.registerTab(this); //!!!
		service = AndroidNTService.getInstance();

		setContentView(R.layout.tabtagactivity);

		/* Getting information concerning the child views */
		txtTag = (EditText) findViewById(R.id.txt_tag);
		txtTag.setFocusable(true);
		txtTag.setFocusableInTouchMode(true);
		txtNumberOfMeasurements = (TextView) findViewById(R.id.txtNumberOfMeasurements);
		txtNumberOfMeasurements.setText(Integer.toString(numberOfMeasurementsToTag));
		txtHowManyQuestion = (TextView) findViewById(R.id.txtHowManyTagQuestion);

		//Tag button
		btnTag = (Button) findViewById(R.id.tagButton);
		btnTag.setOnClickListener(dynamicOnTagListener);

		//+/- buttons
		btnPlus = ((Button) findViewById(R.id.plusButton));
		btnPlus.setOnClickListener(new OnClickListener()
		{
			public void onClick(View v)
			{
				if(numberOfMeasurementsToTag < MAX_TAGGABLE_MEASUREMENTS_SERIES && numberOfMeasurementsToTag < tagQueue.getSize())
					txtNumberOfMeasurements.setText(Integer.toString(++numberOfMeasurementsToTag));
			}
		});
		btnMin = ((Button) findViewById(R.id.minButton));
		btnMin.setOnClickListener(new OnClickListener()
		{
			public void onClick(View v)
			{
				if(numberOfMeasurementsToTag > 1)
					txtNumberOfMeasurements.setText(Integer.toString(--numberOfMeasurementsToTag));
			}
		});

		//Spinner spinner = (Spinner) findViewById(R.id.amount_spinner);
		//ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.amount_of_measurements_array, android.R.layout.simple_spinner_item);
		//adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		//spinner.setAdapter(adapter);
		//spinner.setOnItemSelectedListener(...);
	}

	@Override
	protected void onStart()
	{
		super.onStart();
		(new Handler()).post(new Runnable()
		{
			public void run()
			{
				txtTag.requestFocus();
			}
		});
	}

	/**
	 * Method used to update the tagQueue right after the user clicked the tag-tab
	 */
	public void update(NTMeasurement newMeasurement, Track track)
	{
		// When we haven't stored the last X measurements since we chose to tag some of them...
		if(needsNewTrack == true)
		{
			// ... we need to store them and make sure no new ones will be stored.
			needsNewTrack = false;
			tagQueue = new CyclicQueue(track.getBufferSize());

			Enumeration enumerator = track.getMeasurementsNewestFirst();
			int mnumber = 0;
			if(enumerator.hasMoreElements())
				// We skip the newest element, since this element is the first measurement recorded when we where in the
				// current screen. This element should not be part of the tagging, since the user didn't saw this element
				// yet when he chose to tag an interval.
				enumerator.nextElement();
			while(enumerator.hasMoreElements())
			{
				tagQueue.offer(enumerator.nextElement());
			}
		}
	}

	private OnClickListener dynamicOnTagListener = new OnClickListener()
	{
		//adds the tag that can currently be found to the tag-variable, to the stored measurements.
		public void onClick(View v)
		{
			if(txtTag.getText().toString().equals(""))
				return;
			int toTag = Math.min(tagQueue.getSize(), numberOfMeasurementsToTag);
			for(int i = 0; i < toTag; i++)
			{
				//m is the newest measurement
				NTMeasurement m = (NTMeasurement) tagQueue.serve();
				m.addUserTags(txtTag.getText().toString());
			}
			//TODO store tag (in pref's?) for suggestion dropdown
			Toaster.displayToast("The last " + toTag + " measurements have been tagged with \"" + txtTag.getText().toString() + "\"");
			txtTag.setText(""); //clear field
			mainActivity.showGraph(); //go back to graph tab
		}
	};

	private OnClickListener fixedOnTagListener = new OnClickListener()
	{
		//adds the tag that can currently be found to the tag-variable, to the stored measurements.
		public void onClick(View v)
		{
			if(txtTag.getText().toString().equals(""))
				return;
			//Extra check to make sure we do not try to tag non-existing measurements
			//Note that lastMeasurementLoc refers to the rightmost measurement, which is in time the newest measurement of the selection.
			int lastMLoc = Math.min(numberLastM, service.getTrack().getBufferSize()-1);
			int firstMLoc = numberFirstM;
			Track track = service.getTrack();
			Vector tagsVector = new Vector<String>();
			String tagsString = txtTag.getText().toString();
			String[] tagsArray = tagsString.split(" ");
			if (track!=null)
			{
				for(int i = 0; i < tagsArray.length ; i++)
					tagsVector.add(tagsArray[i]);
				Saver saver = track.getSaver();
				//Calculate offset to the beginning of the track.
				//+1 since in the records measurementcount should start at 1 in stead of 0.
				int firstMLocInTrack = track.getStatistics().getNumMeasurements()-(track.getBufferSize()-firstMLoc) + 1;
				int lastMLocInTrack = track.getStatistics().getNumMeasurements()-(track.getBufferSize()-lastMLoc) + 1;

				saver.save(new TaggedInterval(firstMLocInTrack, lastMLocInTrack, tagsVector));

			}

			//This is only for graphical matters. To display the vertical blue line on the tagged measurements
			//TODO replace graphical notation
			for(int i = tagQueue.getSize(); i > firstMLoc; i--)
			{
				NTMeasurement m = (NTMeasurement) tagQueue.serve();
				if(i > lastMLoc+1)
					continue;
				m.addUserTags(txtTag.getText().toString());
			}

			//TODO store tag (in pref's?) for suggestion dropdown
			//+1 since numberFirstM and numberLastM start counting from 0, while most users feel that counting starting with 1 is more natural
			Toaster.displayToast("Measurements number " + (numberFirstM+1) + " up until number " + (numberLastM+1) + " have been tagged with \"" +
					tagsArray[0] +"\"" + (tagsArray.length > 1 ? " and others" : ""));
			
			txtTag.setText(""); //clear field
			mainActivity.showGraph(); //go back to graph tab
		}
	};

	public boolean mustRemainInformed()
	{
		return false;
	}

	@Override
	protected void onResume()
	{
		super.onResume();
		needsNewTrack = true;
		//Make sure that when tag-tab is used the user can choose how many measurements he wants to tag.
		//E.g. to make sure that the following scenario is not possible:
		//1. Select multiple measurements to tag using touch screen
		//2. Automatically go to tagging tab
		//3. Click on graph tab again without tagging
		//4. Click on tag tab without selecting measurements
		//5. Find that the number of taggable elements is still fixed
		dynamicTagging();
	}

	@Override
	protected void onPause()
	{
		super.onPause();
		needsNewTrack = false;
		//tagQueue = new CyclicQueue();
	}

	public void start()
	{
		txtTag.setEnabled(true);
		btnTag.setEnabled(true);
	}

	public void stop()
	{
		txtTag.setEnabled(false);
		btnTag.setEnabled(false);
		mainActivity.showGraph(); //go back to graph tab
	}

	public void refresh()
	{
		boolean state = service.isMeasuring();
		txtTag.setEnabled(state);
		btnTag.setEnabled(state);
	}

	//Tagtab specific stuff: to tag a set of measurements located in a certain range

	/**
	 * numberFirstM denotes the location of the leftmost selected measurement,
	 * starting from 0 for the leftmost, 1 for the one right next to it, etc.
	 * numberLastM denotes the same but for the location of the rightmost selected measurement
	 */
	public void tagMeasurementsSet(int numberFirstM, int numberLastM){
		this.numberFirstM = numberFirstM;
		this.numberLastM = numberLastM;
		fixedTagging();
	}

	public void fixedTagging()
	{
		numberOfMeasurementsToTag = numberLastM-numberFirstM+1;
		txtNumberOfMeasurements.setText(Integer.toString(numberOfMeasurementsToTag));
		btnMin.setEnabled(false);
		btnPlus.setEnabled(false);
		btnTag.setOnClickListener(fixedOnTagListener);
		txtHowManyQuestion.setText("You are about to tag " + numberOfMeasurementsToTag + " measurements.");
	}

	public void dynamicTagging()
	{
		numberOfMeasurementsToTag = MAX_TAGGABLE_MEASUREMENTS_SERIES;
		txtNumberOfMeasurements.setText(Integer.toString(numberOfMeasurementsToTag));
		btnMin.setEnabled(true);
		btnPlus.setEnabled(true);
		btnTag.setOnClickListener(dynamicOnTagListener);
		txtHowManyQuestion.setText("How many of the previous measurements do you want to tag?");
	}
}
