/**
 * --------------------------------------------------------------------------------
 *  NoiseTube Mobile client (Java implementation; Android version)
 *  
 *  Copyright (C) 2008-2010 SONY Computer Science Laboratory Paris
 *  Portions contributed by Vrije Universiteit Brussel (BrusSense team), 2008-2011
 *  Android port by Vrije Universiteit Brussel (BrusSense team), 2010-2011
 * --------------------------------------------------------------------------------
 *  This library is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU Lesser General Public License, version 2.1, as published
 *  by the Free Software Foundation.
 *  
 *  This library is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 *  details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License along
 *  with this library; if not, write to:
 *    Free Software Foundation, Inc.,
 *    51 Franklin Street, Fifth Floor,
 *    Boston, MA  02110-1301, USA.
 *  
 *  Full GNU LGPL v2.1 text: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt
 *  NoiseTube project source code repository: http://code.google.com/p/noisetube
 * --------------------------------------------------------------------------------
 *  More information:
 *   - NoiseTube project website: http://www.noisetube.net
 *   - Sony Computer Science Laboratory Paris: http://csl.sony.fr
 *   - VUB BrusSense team: http://www.brussense.be
 * --------------------------------------------------------------------------------
 */

package net.noisetube.util;

/**
 * Android-specific replacement of net.noisetube.util.MathNT from shared code
 * (version in shared-code must be excluded from build-path to avoid a clash with this one)
 * 
 * @author mstevens
 * 
 * All methods are marked as final to encourage the compiler to inline calls
 */
public class MathNT
{

	/**
	 * Changed for Android
	 */
	public static final long round(double a)
	{
		return Math.round(a); //use Android's Math.round
	}

	/**
	 * Changed for Android
	 */
	public static final double roundTo(double number, int decimals)
	{
		if(decimals > 0)
		{
			double r = Math.pow(10, decimals); //use Android's Math.pow
			return Math.round(number * r) / r; //use Android's Math.round
		}
		else
			throw new IllegalArgumentException("decimals must be > 0");
	}

	public static final double powerOfTen(int power)
	{
		double r = 1;
		if(power != 0)
			for(int p = 0; p < Math.abs(power); p++)
				r *= 10;
		if(power < 0)
			r = 1 / r;
		return r;
	}

	/**
	 * Changed for Android
	 */
	public static final double atan(double d)
	{
		return Math.atan(d); //use Android's Math.atan
	}
	
	/**
	 * Changed for Android
	 */
	public static final double exp(double d)
	{
		return Math.exp(d); //use Android's Math.exp
	}

	/**
	 * Changed for Android
	 */
	public static final double log(double d)
	{
		return MathNT.log(d); //use Android's Math.log
	}

	/**
	 * Changed for Android
	 */
	public static final double log10(double d)
	{
		return Math.log10(d);
	}
	
	/**
	 * Changed for Android
	 */
	public static final double pow(double x, double y)
	{
		return Math.pow(x, y); //use Android's Math.pow
	}

}
